@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.66 2007/09/17 14:01:27 lav Exp $ */
+/* $Id: CmdExec.h,v 1.69 2009/03/20 10:22:38 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -93,6 +93,8 @@ private:
       cmd_creator_t creator;
       const char  *short_desc;
       const char  *long_desc;
+
+      static int cmp(const CmdExec::cmd_rec *a,const CmdExec::cmd_rec *b);
    };
    static const cmd_rec static_cmd_table[];
    static xarray<cmd_rec> dyn_cmd_table;
@@ -180,6 +182,7 @@ public:
    bool interactive;
    bool top_level;
    bool verbose;
+   bool auto_terminate_in_bg;
    SMTaskRef<StatusLine> status_line;
    void SetCmdFeeder(CmdFeeder *new_feeder);
    void	RemoveFeeder();
@@ -215,6 +218,7 @@ public:
 	 Reconfig(0);
       }
    void SetStatusLine(StatusLine *s) { status_line=s; }
+   void SetAutoTerminateInBackground(bool b) { auto_terminate_in_bg=b; }
 
    static void RegisterCommand(const char *name,cmd_creator_t creator,
       const char *short_name=0,const char *long_name=0);
