@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CopyJob.h,v 1.18 2002/09/02 10:47:36 lav Exp $ */
+/* $Id: CopyJob.h,v 1.28 2009/04/29 13:38:54 lav Exp $ */
 
 #ifndef COPYJOB_H
 #define COPYJOB_H
@@ -29,17 +29,20 @@
 
 class CopyJob : public Job
 {
-   FileCopy *c;
+protected:
+   SMTaskRef<FileCopy> c;
    bool done;
-   char *name; // file name
-   char *dispname; // displayed file name
-   char *op;   // command name
+   xstring_c name; // file name
+   xstring_c dispname; // displayed file name
+   xstring_c op;   // command name
    bool no_status;
    bool no_status_on_write;
    bool clear_status_on_write;
 
    void SetDispName();
 
+   void PrepareToDie();
+
 public:
    CopyJob(FileCopy *c1,const char *n,const char *op1);
    ~CopyJob();
@@ -50,13 +53,13 @@ public:
    bool HasStatus() const { return !no_status; }
 
    int Do();
-   int Done();
+   int Done() { return done; }
    int ExitCode();
 
-   void Suspend() { c->Suspend(); Job::Suspend(); }
-   void Resume() { Job::Resume(); c->Resume(); }
-   void Fg() { c->Fg(); Job::Fg(); }
-   void Bg() { Job::Bg(); c->Bg(); }
+   void SuspendInternal() { c->SuspendSlave(); }
+   void ResumeInternal()  { c->ResumeSlave(); }
+   void Fg() { if(c) c->Fg(); Job::Fg(); }
+   void Bg() { Job::Bg(); if(c) c->Bg(); }
 
    int AcceptSig(int sig);
    pid_t GetProcGroup() { return c?c->GetProcGroup():0; }
@@ -64,7 +67,7 @@ public:
    bool Error() { return c->Error(); }
    const char *ErrorText() { return c->ErrorText(); }
    double GetTimeSpent() { return c->GetTimeSpent(); }
-   off_t GetBytesCount() { return c->GetBytesCount(); }
+   virtual off_t GetBytesCount() { return c->GetBytesCount(); }
    off_t GetSize() { return c->GetSize(); }
    off_t GetPos()  { return c->GetPos(); }
    float GetRate() { return c->GetRate(); }
@@ -72,14 +75,15 @@ public:
    long GetETA(off_t rem) { return c->GetETA(rem); }
    const char *GetETAStrSFromTime(time_t t) { return c->GetETAStrSFromTime(t); }
    void SetRange(off_t s,off_t lim) { c->SetRange(s,lim); }
+   void SetRangeLimit(off_t lim) { return c->SetRangeLimit(lim); }
    void SetDate(time_t d) { c->SetDate(d); }
    void SetSize(off_t s)   { c->SetSize(s); }
-   FileCopy *GetCopy() { return c; }
-   FileCopyPeer *GetPut() { return c->put; }
-   FileCopyPeer *GetGet() { return c->get; }
+   const SMTaskRef<FileCopy>& GetCopy() { return c; }
+   const SMTaskRef<FileCopyPeer>& GetPut() { return c->put; }
+   const SMTaskRef<FileCopyPeer>& GetGet() { return c->get; }
 
    const char *Status(const StatusLine *s,bool base=false);
-   void ShowRunStatus(StatusLine *s);
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
    void	PrintStatus(int,const char *);
 
    const char *GetName() const { return name; }
@@ -88,6 +92,14 @@ public:
 
    static CopyJob *NewGet(FileAccess *f,const char *src,const char *dst);
    static CopyJob *NewPut(FileAccess *f,const char *src,const char *dst);
+
+   static const char *FormatBytesTimeRate(off_t bytes,double time);
+};
+class CopyJobCreator
+{
+public:
+   virtual CopyJob *New(FileCopy *c,const char *n,const char *o) const = 0;
+   virtual ~CopyJobCreator() {}
 };
 
 class ArgV;
@@ -99,19 +111,21 @@ protected:
    int errors;
    int count;
    off_t bytes;
-   float time_spent;
+   double time_spent;
    const char *op;
    bool no_status;
-   char *cwd;
+   xstring_c cwd;
    bool cont;
    bool ascii;
-   ArgV *args;
+   Ref<ArgV> args;
 
    virtual void NextFile() = 0;
 
    void SetCopier(FileCopy *c,const char *n);
    void AddCopier(FileCopy *c,const char *n);
 
+   Ref<CopyJobCreator> cj_new;
+
 public:
    int Do();
    int Done();
@@ -122,11 +136,16 @@ public:
    CopyJobEnv(FileAccess *s,ArgV *a,bool c=false);
    ~CopyJobEnv();
 
+   void SetCopyJobCreator(CopyJobCreator *c) { cj_new=c; }
+
    void SayFinalWithPrefix(const char *p);
    void SayFinal() { SayFinalWithPrefix(""); }
    void	PrintStatus(int,const char *);
 
    void Ascii() { ascii=true; }
+
+   double GetTimeSpent() { return time_spent; }
+   off_t GetBytesCount() { return bytes; }
 };
 
 #endif // COPYJOB_H
