@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: DummyProto.h,v 1.10 2007/04/11 17:49:59 lav Exp $ */
+/* $Id: DummyProto.h,v 1.12 2009/03/20 12:28:04 lav Exp $ */
 
 #ifndef DUMMYPROTO_H
 #define DUMMYPROTO_H
@@ -39,6 +39,9 @@ public:
 
    ListInfo *MakeListInfo(const char *path);
    DirList *MakeDirList(ArgV *);
+
+   DummyProto();
+   ~DummyProto();
 };
 
 class DummyNoProto : public DummyProto
