@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.157 2008/08/05 12:10:08 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.160 2009/04/16 14:10:55 lav Exp $ */
 
 #include <config.h>
 
@@ -159,7 +159,7 @@ void FileAccess::CloseOnExec(int fd)
 void  FileAccess::Open(const char *fn,int mode,off_t offs)
 {
 #ifdef OPEN_DEBUG
-   printf("FA::Open(%s,%d)\n",fn?fn:"NULL",mode);
+   printf("%p->FA::Open(%s,%d)\n",this,fn?fn:"NULL",mode);
 #endif
    if(IsOpen())
       Close();
@@ -275,6 +275,23 @@ void FileAccess::Mkdir(const char *fn,bool allp)
    mkdir_p=allp;
 }
 
+StringSet *FileAccess::MkdirMakeSet()
+{
+   StringSet *set=new StringSet;
+   const char *sl=strchr(file,'/');
+   while(sl)
+   {
+      if(sl>file)
+      {
+	 xstring& tmp=xstring::get_tmp(file,sl-file);
+	 if(tmp.ne(".") && tmp.ne(".."))
+	    set->Append(tmp);
+      }
+      sl=strchr(sl+1,'/');
+   }
+   return set;
+}
+
 bool FileAccess::SameLocationAs(const FileAccess *fa) const
 {
    return SameSiteAs(fa);
