@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.148 2008/08/05 12:10:08 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.153 2009/04/27 07:45:26 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -643,7 +643,13 @@ void FileCopy::LogTransfer()
       range_limit=get->GetPos();
    transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",src,dst,
       (long long)GetRangeStart(),range_limit,
-      Speedometer::GetStr(GetBytesCount()/GetTimeSpent()));
+      Speedometer::GetStr(GetBytesCount()/GetTimeSpent()).get());
+}
+
+void FileCopy::SetRange(off_t s,off_t lim)
+{
+   get->SetRange(s,lim);
+   put->SetRange(s,lim);
 }
 
 
@@ -842,7 +848,7 @@ int FileCopyPeerFA::Do()
 	       {
 		  try_time=session->GetTryTime();
 		  retries=session->GetRetries();
-		  Log::global->Format(10,"try_time=%ld, retries=%d\n",try_time,retries);
+		  Log::global->Format(10,"try_time=%ld, retries=%d\n",(long)try_time,retries);
 		  session->Close();
 		  if(can_seek && seek_pos>0)
 		     Seek(FILE_END);
@@ -1139,7 +1145,7 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
       {
 	 try_time=session->GetTryTime();
 	 retries=session->GetRetries();
-	 Log::global->Format(10,"try_time=%ld, retries=%d\n",try_time,retries);
+	 Log::global->Format(10,"try_time=%ld, retries=%d\n",(long)try_time,retries);
 	 session->Close();
 	 if(can_seek && seek_pos>0)
 	    Seek(FILE_END);
@@ -1334,6 +1340,8 @@ void FileCopyPeerFDStream::Seek_LL()
 
 int FileCopyPeerFDStream::getfd()
 {
+   if(!stream)
+      return -1;
    if(stream->fd!=-1)
       return stream->fd;
    int fd=stream->getfd();
