@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.80 2008/08/05 12:10:09 lav Exp $ */
+/* $Id: FileCopy.h,v 1.82 2009/03/03 07:31:48 lav Exp $ */
 
 /*
   classes defined here:
@@ -228,7 +228,7 @@ public:
    void Ascii() { get->Ascii(); put->Ascii(); }
    void DontFailIfBroken() { fail_if_broken=false; }
    void FailIfCannotSeek() { fail_if_cannot_seek=true; }
-   void SetRange(off_t s,off_t lim) { get->SetRange(s,lim); put->SetRange(s,lim); }
+   void SetRange(off_t s,off_t lim);
    void SetRangeLimit(off_t lim) { get->range_limit=lim; }
    off_t GetRangeStart() { return get->range_start; }
    off_t GetRangeLimit() { return get->range_limit; }
