@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.65 2008/05/22 14:50:30 lav Exp $ */
+/* $Id: FileSet.cc,v 1.68 2009/03/03 13:31:39 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -43,10 +43,6 @@
 #include "IdNameCache.h"
 #include "PatternSet.h"
 
-CDECL_BEGIN
-#include "lstat.h"
-CDECL_END
-
 #define fnum files.count()
 
 void  FileInfo::Merge(const FileInfo& f)
@@ -786,7 +782,7 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    t = NEXT_TOKEN;
    if(!t)
       ERR;
-   if(isdigit(*t))
+   if(isdigit((unsigned char)*t))
    {
       // it's size, so the previous was group:
       fi->SetGroup(group_or_size);
@@ -827,12 +823,9 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    date.tm_hour=date.tm_min=0;
    date.tm_sec=30;
    int prec=30;
-   if(strlen(t)==5)
-   {
-      if(sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min)!=2)
-	 ERR;
+
+   if(sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min)==2)
       date.tm_year=guess_year(date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min) - 1900;
-   }
    else
    {
       if(day_of_month+strlen(day_of_month)+1 == t)
@@ -841,6 +834,7 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
       /* We don't know the hour.  Set it to something other than 0, or
        * DST -1 will end up changing the date. */
       date.tm_hour = 12;
+      date.tm_min=0;
       date.tm_sec=0;
       prec=12*HOUR;
    }
