@@ -1,12 +1,13 @@
-/* lftp and utils
+/*
+ * lftp and utils
  *
- * Copyright (c) 2001-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2001-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * Portions from GNU fileutils.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,7 +20,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
 #include <config.h>
 #include "FileSet.h"
 
@@ -55,7 +55,7 @@ ResDecl	res_default_cls         ("cmd:cls-default",  "-F", FileSetOutput::Valida
 ResDecl res_time_style	("cmd:time-style", "%b %e  %Y|%b %e %H:%M", 0, ResMgr::NoClosure);
 
 /* note: this sorts (add a nosort if necessary) */
-void FileSetOutput::print(FileSet &fs, OutputJob *o) const
+void FileSetOutput::print(FileSet &fs, const JobRef<OutputJob>& o) const
 {
    fs.Sort(sort, sort_casefold, sort_reverse);
    if(sort_dirs_first) fs.Sort(FileSet::DIRSFIRST, false, sort_reverse);
@@ -144,8 +144,8 @@ void FileSetOutput::print(FileSet &fs, OutputJob *o) const
 	 if(!use_fmt || !*use_fmt)
 	    use_fmt="%b %e  %Y\n%b %e %H:%M";
 
-	 char *dt_mem = xstrftime(use_fmt, localtime (&f->date));
-	 char *dt=strtok(dt_mem,"\n|");
+	 xstring_ca dt_mem(xstrftime(use_fmt, localtime (&f->date.ts)));
+	 char *dt=strtok(dt_mem.get_non_const(),"\n|");
 	 if(recent) {
 	    char *dt1=strtok(NULL,"\n|");
 	    if(dt1)
@@ -154,14 +154,11 @@ void FileSetOutput::print(FileSet &fs, OutputJob *o) const
 	 if (!(f->defined&f->DATE)) {
 	    /* put an empty field; make sure it's the same width */
 	    int wid = mbswidth(dt, 0);
-	    xfree(dt_mem); dt_mem=0;
-
 	    dt = string_alloca(wid+1);
 	    memset(dt, ' ', wid);
 	    dt[wid] = 0;
 	 }
 	 c.addf("%s ", "", dt);
-	 xfree(dt_mem);
       }
 
       const char *nm = f->name;
@@ -206,22 +203,6 @@ const char *FileSetOutput::FileInfoSuffix(const FileInfo &fi) const
    return "";
 }
 
-FileSetOutput::FileSetOutput(const FileSetOutput &cp)
-{
-   memset(this, 0, sizeof(*this));
-   *this = cp;
-}
-
-const FileSetOutput &FileSetOutput::operator = (const FileSetOutput &cp)
-{
-   if(this == &cp) return *this;
-
-   memcpy(this, &cp, sizeof(*this));
-   pat = xstrdup(cp.pat);
-   time_fmt = xstrdup(cp.time_fmt);
-   return *this;
-}
-
 void FileSetOutput::config(const OutputJob *o)
 {
    width = o->GetWidth();
@@ -241,19 +222,29 @@ void FileSetOutput::long_list()
       output_block_size = 1;
 }
 
-const char *FileSetOutput::ValidateArgv(char **s)
+const char *FileSetOutput::parse_res(const char *res)
+{
+   Ref<ArgV> arg(new ArgV("",res));
+   const char *error=parse_argv(arg);
+   if(error)
+      return error;
+
+   /* shouldn't be any non-option arguments */
+   if(arg->count() > 1)
+      return _("non-option arguments found");
+
+   return 0;
+}
+
+const char *FileSetOutput::ValidateArgv(xstring_c *s)
 {
    if(!*s) return NULL;
 
-   ArgV arg("", *s);
    FileSetOutput tmp;
 
-   const char *ret = tmp.parse_argv(&arg);
+   const char *ret = tmp.parse_res(*s);
    if(ret) return ret;
 
-   /* shouldn't be any non-option arguments */
-   if(arg.count() > 1) return _("non-option arguments found");
-
    return NULL;
 }
 
@@ -281,32 +272,23 @@ int FileSetOutput::Need() const
 #undef super
 #define super SessionJob
 
-clsJob::clsJob(FA *s, ArgV *a, const FileSetOutput &_opts, OutputJob *_output):
+clsJob::clsJob(FA *s, ArgV *a, FileSetOutput *_opts, OutputJob *_output):
    SessionJob(s),
    fso(_opts),
    args(a),
-   num(1)
+   done(0),
+   use_cache(true),
+   error(false),
+   num(1),
+   state(INIT)
 {
-   use_cache=true;
-   done=0;
-   dir=0;
-   mask=0;
-   state=INIT;
    list_info=0;
 
    if(args->count() == 1)
       args->Add("");
 
    output=_output;
-   output->SetParentFg(this);
-}
-
-clsJob::~clsJob()
-{
-   delete args;
-   xfree(dir);
-   Delete(list_info);
-   Delete(output);
+   AddWaiting(output);
 }
 
 int clsJob::Done()
@@ -329,14 +311,11 @@ int clsJob::Do()
 
    case START_LISTING:
    {
-      Delete(list_info);
       list_info=0;
 
       /* next: */
-      xfree(dir); dir = 0;
-      xfree(mask); mask = 0;
-
-      dir=xstrdup(args->getnext());
+      mask.set(0);
+      dir.set(args->getnext());
       if(!dir) {
 	 /* done */
 	 state=DONE;
@@ -344,27 +323,20 @@ int clsJob::Do()
       }
 
       /* If the basename contains wildcards, set up the mask. */
-      mask = strrchr(dir, '/');
-      if(!mask) mask=dir;
-      if(Glob::HasWildcards(mask)) {
-	 if(mask == dir)
-	    dir = xstrdup("");
-	 else {
-	    /* The mask is the whole argument, not just the basename; this is
-	     * because the whole relative paths will end up in the FileSet, and
-	     * that's what this pattern will be matched against. */
-	    char *newmask = xstrdup(dir);
+      const char *bn = basename_ptr(dir);
+      if(Glob::HasWildcards(bn)) {
+	 /* The mask is the whole argument, not just the basename; this is
+	  * because the whole relative paths will end up in the FileSet, and
+	  * that's what this pattern will be matched against. */
+	 mask.set(dir);
+	 // leave the final / on the path, to prevent the dirname of
+	 // "file/*" from being treated as a file
+	 dir.truncate(bn-dir); // this can result in dir eq ""
+      }
 
-	    // leave the final / on the path, to prevent the dirname of
-	    // "file/*" from being treated as a file
-	    mask[1] = 0;
-	    mask = newmask;
-	 }
-      } else mask=0;
-
-      list_info=new GetFileInfo(session, dir, fso.list_directories);
+      list_info=new GetFileInfo(session, dir, fso->list_directories);
       list_info->UseCache(use_cache);
-      list_info->Need(fso.Need());
+      list_info->Need(fso->Need());
 
       state=GETTING_LIST_INFO;
       m=MOVED;
@@ -376,18 +348,19 @@ int clsJob::Do()
 
       if(list_info->Error()) {
 	 eprintf("%s\n", list_info->ErrorText());
+	 error=true;
 	 state=START_LISTING;
 	 return MOVED;
       }
 
       /* one just finished */
-      fso.pat = mask;
+      fso->pat.set_allocated(mask.borrow());
       FileSet *res = list_info->GetResult();
 
       if(res)
-	 fso.print(*res, output);
+	 fso->print(*res, output);
 
-      fso.pat = 0;
+      fso->pat.set(0);
       delete res;
 
       state=START_LISTING;
@@ -420,9 +393,9 @@ void clsJob::ResumeInternal()
    session->ResumeSlave();
 }
 
-void clsJob::ShowRunStatus(StatusLine *s)
+void clsJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
-   if(fso.quiet)
+   if(fso->quiet)
       return;
 
    if(!output->ShowStatusLine(s))
