@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpDirList.cc,v 1.25 2006/02/06 10:57:25 lav Exp $ */
+/* $Id: FtpDirList.cc,v 1.31 2008/11/27 05:56:16 lav Exp $ */
 
 #include <config.h>
 
@@ -56,7 +56,7 @@ int FtpDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,&err,
+      if(use_cache && FileAccess::cache->Find(session,pattern,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
 	 if(err)
@@ -72,8 +72,8 @@ int FtpDirList::Do()
       {
 	 session->Open(pattern,FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -83,7 +83,7 @@ int FtpDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-      LsCache::Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
+      FileAccess::cache->Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
       return MOVED;
    }
 
@@ -119,20 +119,6 @@ int FtpDirList::Do()
    return m;
 }
 
-FtpDirList::FtpDirList(ArgV *a,FileAccess *fa)
-   : DirList(a)
-{
-   session=fa;
-   ubuf=0;
-   pattern=args->Combine(1);
-}
-
-FtpDirList::~FtpDirList()
-{
-   Delete(ubuf);
-   xfree(pattern);
-}
-
 const char *FtpDirList::Status()
 {
    static char s[256];
@@ -231,7 +217,11 @@ bool FtpDirList::TryColor(const char *line_c,int len)
       group[0]=0;
       n=sscanf(line,"%11s %d %31s %lld %3s %2d %5s%n",perms,&nlink,
 	    user,&size,month_name,&day,year_or_time,&consumed);
+      if(n!=7)
+	 return false;
    }
+   else if(n!=8)
+      return false;
    if(consumed>0 && -1!=(parse_perms(perms+1))
    && -1!=(parse_month(month_name))
    && -1!=parse_year_or_time(year_or_time,&year,&hour,&minute)
