@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.h,v 1.32 2008/08/05 12:10:10 lav Exp $ */
+/* $Id: Job.h,v 1.35 2008/11/27 05:56:20 lav Exp $ */
 
 #ifndef JOB_H
 #define JOB_H
@@ -32,6 +32,8 @@
 #include "fg.h"
 #include "FileAccess.h"
 
+#define JobRef SMTaskRef // it is basically the same
+
 class Job : public SMTask
 {
    static void SortJobs();
@@ -41,8 +43,9 @@ class Job : public SMTask
 
 protected:
    bool fg;
-   FgData *fg_data;
+   Ref<FgData> fg_data;
 
+   void PrepareToDie();
    virtual ~Job();
 
 public:
@@ -52,6 +55,7 @@ public:
    xarray<Job*> waiting;
 
    void AddWaiting(Job *);
+   template<class T> void AddWaiting(const JobRef<T>& r) { AddWaiting(r.get_non_const()); }
    void RemoveWaiting(const Job *);
    void ReplaceWaiting(Job *from,Job *to);
 
@@ -154,6 +158,5 @@ public:
    void Bg();
 };
 
-#define JobRef SMTaskRef // it is basically the same
 
 #endif /* JOB_H */
