@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.51.2.1 2007/03/26 07:00:39 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.63 2008/11/27 05:56:21 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -27,16 +27,21 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
-#include <glob.h>
 #include <utime.h>
 
 #include "LocalAccess.h"
-#include "xmalloc.h"
 #include "xstring.h"
 #include "misc.h"
 #include "log.h"
 #include "LocalDir.h"
 
+CDECL_BEGIN
+#include <glob.h>
+CDECL_END
+
+#define FILES_AT_ONCE_STAT 64
+#define FILES_AT_ONCE_READDIR 256
+
 FileAccess *LocalAccess::New() { return new LocalAccess(); }
 
 void LocalAccess::ClassInit()
@@ -49,40 +54,31 @@ void LocalAccess::Init()
 {
    done=false;
    error_code=OK;
-   stream=0;
    home.Set(getenv("HOME"));
-   xfree(hostname);
-   hostname=xstrdup("localhost");
+   hostname.set("localhost");
 }
 
 LocalAccess::LocalAccess() : FileAccess()
 {
    Init();
-   char *c=xgetcwd();
-   cwd.Set(c?c:".");
-   xfree(c);
+   xstring_ca c(xgetcwd());
+   cwd.Set(c?c.get():".");
 }
 LocalAccess::LocalAccess(const LocalAccess *o) : FileAccess(o)
 {
    Init();
 }
-LocalAccess::~LocalAccess()
-{
-   delete stream;
-}
 
 void LocalAccess::errno_handle()
 {
    int e=errno;
-   xfree(error);
    const char *err=strerror(e);
-   error=(char*)xmalloc(xstrlen(file)+xstrlen(file1)+strlen(err)+20);
    if(mode==RENAME)
-      sprintf(error,"rename(%s, %s): %s",file,file1,err);
+      error.vset("rename(",file.get(),", ",file1.get(),"): ",err,NULL);
    else
-      sprintf(error,"%s: %s",file,err);
+      error.vset(file.get(),": ",err,NULL);
    if(e!=EEXIST)
-      Log::global->Format(0,"**** %s\n",error);
+      LogError(0,"%s",error.get());
 }
 
 int LocalAccess::Done()
@@ -122,20 +118,20 @@ int LocalAccess::Do()
 	 if(mode==LIST)
 	 {
 	    if(file && file[0])
-	       sprintf(cmd,"ls %s",file);
+	       sprintf(cmd,"ls %s",file.get());
 	    else
 	       strcpy(cmd,"ls");
 	 }
 	 else if(mode==LONG_LIST)
 	 {
 	    if(file && file[0])
-	       sprintf(cmd,"ls -l %s",file);
+	       sprintf(cmd,"ls -l %s",file.get());
 	    else
 	       strcpy(cmd,"ls -la");
 	 }
 	 else// if(mode==QUOTE_CMD)
 	    strcpy(cmd,file);
-	 DebugPrint("---- ",cmd,5);
+	 LogNote(5,"running `%s'",cmd);
 	 InputFilter *f_stream=new InputFilter(cmd);
 	 f_stream->SetCwd(cwd);
 	 stream=f_stream;
@@ -179,7 +175,7 @@ int LocalAccess::Do()
    }
    case(REMOVE): {
       const char *f=dir_file(cwd,file);
-      Log::global->Format(5,"---- remove(%s)\n",f);
+      LogNote(5,"remove(%s)",f);
       if(remove(f)==-1)
       {
 	 errno_handle();
@@ -198,28 +194,24 @@ int LocalAccess::Do()
       return MOVED;
    case(RENAME):
    {
-      char *cwd_file1=xstrdup(dir_file(cwd,file1));
+      const char *cwd_file1=dir_file(cwd,file1);
+      cwd_file1=alloca_strdup(cwd_file1); // save it
       if(rename(dir_file(cwd,file),cwd_file1)==-1)
       {
 	 errno_handle();
 	 error_code=NO_FILE;
       }
-      xfree(cwd_file1);
       done=true;
       return MOVED;
    }
    case(MAKE_DIR):
       if(mkdir_p)
       {
-	 char *sl=strchr(file,'/');
+	 const char *sl=strchr(file,'/');
 	 while(sl)
 	 {
 	    if(sl>file)
-	    {
-	       *sl=0;
-	       mkdir(dir_file(cwd,file),0775);
-	       *sl='/';
-	    }
+	       mkdir(dir_file(cwd,xstring::get_tmp(file,sl-file)),0775);
 	    sl=strchr(sl+1,'/');
 	 }
       }
@@ -498,7 +490,6 @@ int LocalAccess::StoreStatus()
       if(stream->error())
 	 SetError(NO_FILE,stream->error_text);
    }
-   delete stream;
    stream=0;
    if(error_code==OK && entity_date!=NO_DATE)
    {
@@ -517,7 +508,6 @@ void LocalAccess::Close()
 {
    done=false;
    error_code=OK;
-   delete stream;
    stream=0;
    FileAccess::Close();
 }
@@ -541,6 +531,16 @@ bool LocalAccess::SameLocationAs(const FileAccess *fa) const
    return !xstrcmp(cwd,o->cwd);
 }
 
+class LocalListInfo : public ListInfo
+{
+   DIR *dir;
+public:
+   LocalListInfo(FileAccess *s,const char *d) : ListInfo(s,d), dir(0) {}
+   ~LocalListInfo() { if(dir) closedir(dir); }
+   const char *Status();
+   int Do();
+};
+
 ListInfo *LocalAccess::MakeListInfo(const char *path)
 {
    return new LocalListInfo(this,path);
@@ -548,48 +548,72 @@ ListInfo *LocalAccess::MakeListInfo(const char *path)
 
 int LocalListInfo::Do()
 {
+   int m=STALL;
+
    if(done)
       return STALL;
 
-   const char *dir=session->GetCwd();
-   DIR *d=opendir(dir);
-   struct dirent *f;
-
-   if(d==0)
+   if(!dir && !result)
    {
-      const char *err=strerror(errno);
-      char *mem=(char*)alloca(strlen(err)+strlen(dir)+3);
-      sprintf(mem,"%s: %s",dir,err);
-      SetError(mem);
-      return MOVED;
+      const char *path=session->GetCwd();
+      dir=opendir(path);
+      if(!dir)
+      {
+	 SetError(xstring::format("%s: %s",path,strerror(errno)));
+	 return MOVED;
+      }
+      m=MOVED;
    }
-
-   result=new FileSet;
-   for(;;)
+   if(dir)
    {
-      f=readdir(d);
-      if(f==0)
-	 break;
-      const char *name=f->d_name;
-      if(name[0]=='~')
-	 name=dir_file(".",name);
-      result->Add(new FileInfo(name));
+      if(!result)
+	 result=new FileSet;
+      int count=FILES_AT_ONCE_READDIR;
+      for(;;)
+      {
+	 struct dirent *f=readdir(dir);
+	 if(f==0)
+	    break;
+	 const char *name=f->d_name;
+	 if(name[0]=='~')
+	    name=dir_file(".",name);
+	 result->Add(new FileInfo(name));
+	 if(!--count)
+	    return MOVED;	// let other tasks run
+      }
+      closedir(dir);
+      dir=0;
+      result->rewind();
+      m=MOVED;
    }
-   closedir(d);
-
-   result->rewind();
-   for(FileInfo *file=result->curr(); file!=0; file=result->next())
+   if(!dir && result)
    {
-      const char *name=dir_file(dir,file->name);
-      file->LocalFile(name, follow_symlinks);
-      if(!(file->defined&file->TYPE))
-	 result->SubtractCurr();
+      int count=FILES_AT_ONCE_STAT;
+      const char *path=session->GetCwd();
+      for(FileInfo *file=result->curr(); file!=0; file=result->next())
+      {
+	 const char *name=dir_file(path,file->name);
+	 file->LocalFile(name, follow_symlinks);
+	 if(!(file->defined&file->TYPE))
+	    result->SubtractCurr();
+	 if(!--count)
+	    return MOVED;	// let other tasks run
+      }
+      result->Exclude(exclude_prefix,exclude);
+      done=true;
+      m=MOVED;
    }
-
-   result->Exclude(exclude_prefix,exclude);
-
-   done=true;
-   return MOVED;
+   return m;
+}
+const char *LocalListInfo::Status()
+{
+   if(done)
+      return "";
+   if(dir && result)
+      return xstring::format("%s (%d)",_("Getting directory contents"),result->count());
+   if(result && result->count())
+      return xstring::format("%s (%d%%)",_("Getting files information"),result->curr_pct());
+   return "";
 }
 
 #include "FileGlob.h"
@@ -598,19 +622,18 @@ class LocalGlob : public Glob
    const char *cwd;
 public:
    LocalGlob(const char *cwd,const char *pattern);
-   const char *Status() { return "..."; }
+   const char *Status() { return "Reading directory"; }
    int Do();
 };
 Glob *LocalAccess::MakeGlob(const char *pattern)
 {
-   xfree(file);
-   file=xstrdup(pattern);
+   file.set(pattern);
    ExpandTildeInCWD();
    return new LocalGlob(cwd,file);
 }
 
 LocalGlob::LocalGlob(const char *c,const char *pattern)
-   : Glob(pattern)
+   : Glob(0,pattern)
 {
    cwd=c;
 }
@@ -630,10 +653,7 @@ int LocalGlob::Do()
    }
    if(chdir(cwd)==-1)
    {
-      const char *se=strerror(errno);
-      char *err=(char*)alloca(strlen(cwd)+strlen(se)+20);
-      sprintf(err,"chdir(%s): %s",cwd,se);
-      SetError(err);
+      SetError(xstring::format("chdir(%s): %s",cwd,strerror(errno)));
       return MOVED;
    }
 
@@ -667,6 +687,16 @@ int LocalGlob::Do()
    return MOVED;
 }
 
+class LocalDirList : public DirList
+{
+   SMTaskRef<IOBuffer> ubuf;
+   Ref<FgData> fg_data;
+public:
+   LocalDirList(ArgV *a,const char *cwd);
+   const char *Status() { return ""; }
+   int Do();
+};
+
 DirList *LocalAccess::MakeDirList(ArgV *a)
 {
    return new LocalDirList(a,cwd);
@@ -674,9 +704,8 @@ DirList *LocalAccess::MakeDirList(ArgV *a)
 
 #include "ArgV.h"
 LocalDirList::LocalDirList(ArgV *a,const char *cwd)
-   : DirList(0)
+   : DirList(0,0)
 {
-   fg_data=0;
    a->setarg(0,"ls");
    a->insarg(1,"-l");
    InputFilter *f=new InputFilter(a); // a is consumed.
@@ -715,11 +744,6 @@ int LocalDirList::Do()
    ubuf->Skip(len);
    return MOVED;
 }
-LocalDirList::~LocalDirList()
-{
-   delete fg_data;
-   Delete(ubuf);
-}
 
 #include "modconfig.h"
 #ifdef MODULE_PROTO_FILE
