@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.164 2008/07/23 08:46:10 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.169 2009/05/13 08:40:18 lav Exp $ */
 
 #include <config.h>
 
@@ -41,10 +41,6 @@
 #include "pgetJob.h"
 #include "log.h"
 
-CDECL_BEGIN
-#include "lstat.h"
-CDECL_END
-
 #define set_state(s) do { state=(s); \
    Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 #define waiting_num waiting.count()
@@ -307,7 +303,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 if(target_is_local)
 	    dst_peer=new FileCopyPeerFDStream(new FileStream(target_name,O_WRONLY|O_CREAT|(cont_this?0:O_TRUNC)),FileCopyPeer::PUT);
 	 else
-	    dst_peer=new FileCopyPeerFA(target_session,file->name,FA::STORE);
+	    dst_peer=new FileCopyPeerFA(target_session->Clone(),file->name,FA::STORE);
 
 	 FileCopy *c=FileCopy::New(src_peer,dst_peer,cont_this);
 	 if(remove_source_files)
@@ -323,7 +319,6 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    cp->SetSize(file->size);
 	 AddWaiting(cp);
 	 transfer_count++;
-	 cp->SetParentFg(this);
 	 cp->cmdline.vset("\\transfer ",file->name.get(),NULL);
 
 	 set_state(WAITING_FOR_TRANSFER);
@@ -379,7 +374,6 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    source_session->Clone(),target_session->Clone(),
 	    source_name,target_name);
 	 AddWaiting(mj);
-	 mj->SetParentFg(this);
 	 mj->cmdline.vset("\\mirror ",file->name.get(),NULL);
 
 	 // inherit flags and other things
@@ -916,7 +910,6 @@ int   MirrorJob::Do()
 	    args->Append(file->name);
 	    args->seek(1);
 	    rmJob *j=new rmJob(target_session->Clone(),args);
-	    j->SetParentFg(this);
 	    j->cmdline.set_allocated(args->Combine());
 	    AddWaiting(j);
 	    transfer_count++;
@@ -1005,7 +998,6 @@ int   MirrorJob::Do()
 				 file->mode&~mode_mask,a);
 	    AddWaiting(cj);
 	    transfer_count++;
-	    cj->SetParentFg(this);
 	    cj->cmdline.set_allocated(a->Combine());
 	    cj->BeQuiet(); // chmod is not supported on all servers; be quiet.
 	    m=MOVED;
@@ -1046,7 +1038,6 @@ int   MirrorJob::Do()
 	 if(stats.HaveSomethingDone(flags) && on_change)
 	 {
 	    CmdExec *exec=new CmdExec(source_session->Clone(),0);
-	    exec->SetParentFg(this);
 	    AddWaiting(exec);
 	    exec->FeedCmd(on_change);
 	    exec->FeedCmd("\n");
@@ -1286,6 +1277,7 @@ char *MirrorJob::SetScriptFile(const char *n)
    if(strcmp(n,"-"))
    {
       script=fopen(n,"w");
+      setvbuf(script,NULL,_IOLBF,0);
       script_needs_closing=true;
    }
    else
