@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.104 2008/03/05 06:36:04 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.106 2008/11/27 05:56:23 lav Exp $ */
 
 #include <config.h>
 
@@ -167,10 +167,6 @@ void NetAccess::SetSocketMaxseg(int sock)
 #endif
 }
 
-#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
-CDECL int inet_aton(const char *,struct in_addr *);
-#endif
-
 int NetAccess::SocketCreate(int af,int type,int proto)
 {
    int s=socket(af,type,proto);
@@ -183,7 +179,7 @@ int NetAccess::SocketCreate(int af,int type,int proto)
    if(af==AF_INET)
    {
       b=ResMgr::Query("net:socket-bind-ipv4",hostname);
-      if(!(b && b[0] && inet_aton(b,&bind_addr.in.sin_addr)))
+      if(!(b && b[0] && inet_pton(af,b,&bind_addr.in.sin_addr)))
 	 b=0;
    }
 #if INET6
