@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.42 2006/06/15 14:18:40 lav Exp $ */
+/* $Id: NetAccess.h,v 1.50 2008/11/27 05:56:23 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -26,15 +26,15 @@
 #include "FileAccess.h"
 #include "Resolver.h"
 #include "LsCache.h"
+#include "RateLimit.h"
 
 class NetAccess : public FileAccess
 {
 protected:
-   Resolver *resolver;
+   SMTaskRef<Resolver> resolver;
 
-   sockaddr_u *peer;
-   int	 peer_num;
-   int	 peer_curr;
+   xarray<sockaddr_u> peer;
+   int peer_curr;
    void	 ClearPeer();
    void	 NextPeer();
 
@@ -54,7 +54,7 @@ protected:
    int	 connection_limit;
    bool	 connection_takeover;
 
-   class RateLimit *rate_limit;
+   Ref<RateLimit> rate_limit;
 
    int	 socket_buffer;
    int	 socket_maxseg;
@@ -66,22 +66,22 @@ protected:
    static void MaximizeThroughput(int sock);
    static void ReuseAddress(int sock);
    static int SocketBuffered(int sock);
-   static const char *SocketNumericAddress(const sockaddr_u *u);
-   static int SocketPort(const sockaddr_u *u);
-   static socklen_t SocketAddrLen(const sockaddr_u *u);
+   static const char *SocketNumericAddress(const sockaddr_u *u) { return u->address(); }
+   static int SocketPort(const sockaddr_u *u) { return u->port(); }
+   static socklen_t SocketAddrLen(const sockaddr_u *u) { return u->addr_len(); }
    static int SocketConnect(int fd,const sockaddr_u *u);
    int SocketCreate(int,int,int);
    int SocketCreateTCP(int);
    int Poll(int fd,int ev);
    int CheckHangup(const struct pollfd *pfd,int num);
 
-   char	 *proxy;
-   char	 *proxy_port;
-   char  *proxy_user;
-   char  *proxy_pass;
-   char  *proxy_proto;
+   xstring_c proxy;
+   xstring_c proxy_port;
+   xstring_c proxy_user;
+   xstring_c proxy_pass;
+   xstring_c proxy_proto;
 
-   char	 *home_auto;
+   xstring_c home_auto;
    void	 PropagateHomeAuto();
    const char *FindHomeAuto();
 
@@ -94,7 +94,8 @@ protected:
 
    const char *DelayingMessage();
    bool ReconnectAllowed();
-   bool NextTry();	// returns false if max-retries exceeded.
+   bool CheckRetries();	// returns false if max-retries exceeded.
+   bool NextTry();	// increments retries; does CheckRetries().
    void TrySuccess();	// reset retry counters.
 
    virtual void HandleTimeout();
@@ -118,54 +119,13 @@ public:
    static void ClassInit();
 };
 
-class RateLimit
-{
-public:
-   class BytesPool
-   {
-      friend class RateLimit;
-
-      int pool;
-      int rate;
-      int pool_max;
-      Time t;
-
-      void AdjustTime();
-      void Reset();
-      void Used(int);
-   };
-
-private:
-   static int total_xfer_number;
-   static bool total_reconfig_needed;
-   static void ReconfigTotal();
-   static BytesPool total[2];
-   BytesPool one[2];
-
-public:
-   RateLimit(const char *closure);
-   ~RateLimit();
-
-   enum dir_t { GET=0, PUT=1 };
-
-   int BytesAllowed(dir_t how);
-   int BytesAllowedToGet() { return BytesAllowed(GET); }
-   int BytesAllowedToPut() { return BytesAllowed(PUT); }
-   void BytesUsed(int,dir_t);
-   void BytesGot(int b) { BytesUsed(b,GET); }
-   void BytesPut(int b) { BytesUsed(b,PUT); }
-
-   void Reconfig(const char *name,const char *c);
-};
-
 class GenericParseListInfo : public ListInfo
 {
 protected:
-   FA::fileinfo *get_info;
-   int get_info_cnt;
+   xarray<FA::fileinfo> get_info;
 
    int mode;
-   IOBuffer *ubuf;
+   SMTaskRef<IOBuffer> ubuf;
 
    bool get_time_for_dirs;
    bool can_get_prec_time;
@@ -175,10 +135,8 @@ protected:
 
 public:
    GenericParseListInfo(FileAccess *session,const char *path);
-   virtual ~GenericParseListInfo();
    int Do();
    const char *Status();
 };
 
-
 #endif//NETACCESS_H
