@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.69 2007/08/08 06:23:54 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.74 2008/11/27 05:56:26 lav Exp $ */
 
 #include <config.h>
 
@@ -32,8 +32,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
+#include <limits.h>
 CDECL_BEGIN
-#include <regex.h>
+#include "regex.h"
 CDECL_END
 #include "ResMgr.h"
 #include "SMTask.h"
@@ -411,60 +412,121 @@ const char *ResMgr::TriBoolValidate(xstring_c *value)
    return 0;
 }
 
+static const char power_letter[] =
+{
+  0,	/* not used */
+  'K',	/* kibi ('k' for kilo is a special case) */
+  'M',	/* mega or mebi */
+  'G',	/* giga or gibi */
+  'T',	/* tera or tebi */
+  'P',	/* peta or pebi */
+  'E',	/* exa or exbi */
+  'Z',	/* zetta or 2**70 */
+  'Y'	/* yotta or 2**80 */
+};
+static unsigned long long get_power_multiplier(char p)
+{
+   const char *scan=power_letter;
+   int scale=1024;
+   unsigned long long mul=1;
+   p=toupper(p);
+   while(scan<power_letter+sizeof(power_letter)) {
+      if(p==*scan)
+	 return mul;
+      mul*=scale;
+      scan++;
+   }
+   return 0;
+}
+
 const char *ResMgr::NumberValidate(xstring_c *value)
 {
    const char *v=*value;
+   const char *end=v;
 
-   v+=strspn(v," \t");
+   (void)strtoll(v,const_cast<char**>(&end),0);
+   unsigned long long m=get_power_multiplier(*end);
 
-   if(*v=='-')
-      v++;
-
-   int n=strspn(v,"1234567890");
-
-   if(n==0)
+   if(v==end || m==0 || end[m>1])
       return _("invalid number");
 
-   value->truncate(n+(v-*value));
-
    return 0;
 }
-
 const char *ResMgr::FloatValidate(xstring_c *value)
 {
    const char *v=*value;
+   const char *end=v;
 
-   int n=0;
+   (void)strtod(v,const_cast<char**>(&end));
+   unsigned long long m=get_power_multiplier(*end);
 
-   double f;
-   if(1>sscanf(v,"%lf%n",&f,&n))
+   if(v==end || m==0 || end[m>1])
       return _("invalid floating point number");
 
-   value->truncate(n);
-
    return 0;
 }
-
 const char *ResMgr::UNumberValidate(xstring_c *value)
 {
    const char *v=*value;
+   const char *end=v;
 
-   v+=strspn(v," \t");
-   value->set(v); // drop leading space
+   (void)strtoull(v,const_cast<char**>(&end),0);
+   unsigned long long m=get_power_multiplier(*end);
 
-   v=*value;
-   if(!strncasecmp(v,"0x",2))
-      v+=2;
-
-   int n=strspn(v,"1234567890");
-
-   if(n==0)
-      return _("invalid number");
-
-   value->truncate(n+(v-*value));
+   if(!isdigit((unsigned char)v[0])
+   || v==end || m==0 || end[m>1])
+      return _("invalid unsigned number");
 
    return 0;
 }
+unsigned long long ResValue::to_unumber(unsigned long long max)
+{
+   const char *end=s;
+   unsigned long long v=strtoull(s,const_cast<char**>(&end),0);
+   unsigned long long m=get_power_multiplier(*end);
+   unsigned long long vm=v*m;
+   if(vm/m!=v || vm>max)
+      return max;
+   return vm;
+}
+long long ResValue::to_number(long long min,long long max)
+{
+   const char *end=s;
+   long long v=strtoull(s,const_cast<char**>(&end),0);
+   long long m=get_power_multiplier(*end);
+   long long vm=v*m;
+   if(vm/m!=v)
+      return v>0?max:min;
+   if(vm>max)
+      return max;
+   if(vm<min)
+      return min;
+   return vm;
+}
+ResValue::operator int()
+{
+   return to_number(INT_MIN,INT_MAX);
+}
+ResValue::operator long()
+{
+   return to_number(LONG_MIN,LONG_MAX);
+}
+ResValue::operator unsigned()
+{
+   return to_unumber(UINT_MAX);
+}
+ResValue::operator unsigned long()
+{
+   return to_unumber(ULONG_MAX);
+}
+
+ResMgr::Resource::Resource(Resource *next,const ResType *type,const char *closure,const char *value)
+   : type(type), value(value), closure(closure), next(next)
+{
+}
+ResMgr::Resource::~Resource()
+{
+}
 
 bool ResMgr::Resource::ClosureMatch(const char *cl_data)
 {
@@ -767,16 +829,12 @@ const char *ResMgr::ERegExpValidate(xstring_c *s)
    return 0;
 }
 
-#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
-CDECL int inet_aton(const char *,struct in_addr *);
-#endif
-
 const char *ResMgr::IPv4AddrValidate(xstring_c *value)
 {
    if(!**value)
       return 0;
    struct in_addr addr;
-   if(!inet_aton(*value,&addr))
+   if(!inet_pton(AF_INET,*value,&addr))
       return _("Invalid IPv4 numeric address");
    return 0;
 }
@@ -871,6 +929,14 @@ const char *ResMgr::UNumberPairValidate(xstring_c *value)
 
    return 0;
 }
+void ResValue::ToNumberPair(int &a,int &b)
+{
+   switch(sscanf(s,"%d%*c%d",&a,&b))
+   {
+   case 0: a=0;
+   case 1: b=a;
+   }
+}
 
 ResClient *ResClient::chain;
 ResValue ResClient::Query(const char *name,const char *closure) const
@@ -899,3 +965,16 @@ void ResClient::ReconfigAll(const char *r)
    ListScan(ResClient,chain,next)
       scan->Reconfig(r);
 }
+
+bool ResType::QueryBool(const char *closure) const
+{
+   return Query(closure).to_bool();
+}
+bool ResMgr::QueryBool(const char *name,const char *closure)
+{
+   return Query(name,closure).to_bool();
+}
+bool ResClient::QueryBool(const char *name,const char *closure) const
+{
+   return Query(name,closure).to_bool();
+}
