@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.53 2008/05/16 08:45:00 lav Exp $ */
+/* $Id: ResMgr.h,v 1.55 2008/11/27 05:56:26 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -147,35 +147,20 @@ public:
       {
 	 return ResMgr::str2bool(s);
       }
-   operator int()
-      {
-	 return strtol(s,0,0);
-      }
-   operator long()
-      {
-	 return strtol(s,0,0);
-      }
-   operator double()
-      {
-	 return atof(s);
-      }
-   operator float()
-      {
-	 return atof(s);
-      }
+   unsigned long long to_unumber(unsigned long long max);
+   long long to_number(long long min,long long max);
+   operator int();
+   operator long();
+   operator unsigned();
+   operator unsigned long();
+   operator double() { return atof(s); }
+   operator float()  { return atof(s); }
    operator const char*()
       {
 	 return s;
       }
    bool is_nil() { return s==0; }
-   void ToNumberPair(int &a,int &b)
-      {
-	 switch(sscanf(s,"%d%*c%d",&a,&b))
-	 {
-	 case 0: a=0;
-	 case 1: b=a;
-	 }
-      }
+   void ToNumberPair(int &a,int &b);
 };
 
 class TimeIntervalR : public TimeInterval
