@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.81 2008/05/22 13:20:38 lav Exp $ */
+/* $Id: Resolver.cc,v 1.84 2009/01/27 14:43:12 lav Exp $ */
 
 #include <config.h>
 
@@ -291,7 +291,7 @@ void Resolver::MakeErrMsg(const char *f)
    done=true;
 }
 
-void Resolver::AddAddress(int family,const char *address,int len)
+void Resolver::AddAddress(int family,const char *address,int len, unsigned int scope)
 {
    sockaddr_u add;
    memset(&add,0,sizeof(add));
@@ -315,9 +315,12 @@ void Resolver::AddAddress(int family,const char *address,int len)
          return;
       memcpy(&add.in6.sin6_addr,address,len);
       add.in6.sin6_port=port_number;
-#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
+      add.in6.sin6_scope_id=scope;
+# endif
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       add.sa.sa_len=sizeof(add.in6);
-#endif
+# endif
       break;
 #endif
 
@@ -693,6 +696,7 @@ void Resolver::LookupOne(const char *name)
       struct sockaddr_in6   *inet6_addr;
       const char	    *addr_data;
       int		    addr_len;
+      unsigned int          addr_scope;
 
       memset(&a_hint, 0, sizeof(a_hint));
       a_hint.ai_flags	    = AI_PASSIVE;
@@ -711,6 +715,7 @@ void Resolver::LookupOne(const char *name)
 		  continue;
 
 	       sockname	= a_res->ai_addr;
+	       addr_scope = 0;
 
 	       switch(a_res->ai_family)
 	       {
@@ -722,12 +727,15 @@ void Resolver::LookupOne(const char *name)
 	       case AF_INET6:
 		  inet6_addr  = (sockaddr_in6 *)sockname;
 		  addr_data   = (const char *)&(inet6_addr->sin6_addr.s6_addr);
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
+		  addr_scope  = inet6_addr->sin6_scope_id;
+#endif
 		  addr_len    = sizeof(inet6_addr->sin6_addr.s6_addr);
 		  break;
 	       default:
 		  continue;
 	       }
-	       AddAddress(a_res->ai_family, addr_data, addr_len);
+	       AddAddress(a_res->ai_family, addr_data, addr_len, addr_scope);
 	    }
 	 }
 
@@ -773,7 +781,7 @@ void Resolver::LookupOne(const char *name)
       {
 	 const char * const *a;
 	 for(a=ha->h_addr_list; *a; a++)
-	    AddAddress(ha->h_addrtype, *a, ha->h_length);
+	    AddAddress(ha->h_addrtype, *a, ha->h_length, 0);
 	 retries=0;
 	 af_index++;
 # if defined(HAVE_GETIPNODEBYNAME)
