@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.80 2008/05/16 08:45:01 lav Exp $ */
+/* $Id: SFtp.cc,v 1.85 2009/05/14 08:36:39 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -606,8 +606,7 @@ void SFtp::SendRequest()
 	 SetError(NOT_SUPP);
 	 break;
       }
-      char *file1_wire_path=alloca_strdup(WirePath(file1));
-      SendRequest(new Request_RENAME(WirePath(file),file1_wire_path,
+      SendRequest(new Request_RENAME(WirePath(file),WirePath(file1),
 			SSH_FXF_RENAME_NATIVE,protocol_version),Expect::DEFAULT);
       state=WAITING;
       break;
@@ -622,6 +621,12 @@ void SFtp::SendRequest()
       break;
    }
    case MAKE_DIR:
+      if(mkdir_p)
+      {
+	 Ref<StringSet> dirs(MkdirMakeSet());
+	 for(int i=0; i<dirs->Count(); i++)
+	    SendRequest(new Request_MKDIR(WirePath(dirs->String(i)),protocol_version),Expect::IGNORE);
+      }
       SendRequest(new Request_MKDIR(WirePath(file),protocol_version),Expect::DEFAULT);
       state=WAITING;
       break;
@@ -1000,6 +1005,9 @@ int SFtp::HandleReplies()
    if(state!=CONNECTING_2)
       m|=HandlePty();
 
+   if(!recv_buf)
+      return MOVED;
+
    int i=0;
    Expect *ooo_scan=ooo_chain;
    while(ooo_scan)
@@ -1912,7 +1920,7 @@ void SFtp::Request_RENAME::Pack(Buffer *b)
 
 const char *SFtp::utf8_to_lc(const char *s)
 {
-   if(!recv_translate)
+   if(!recv_translate || !s)
       return s;
 
    recv_translate->ResetTranslation();
@@ -1921,11 +1929,11 @@ const char *SFtp::utf8_to_lc(const char *s)
    int len;
    recv_translate->Get(&s,&len);
    recv_translate->Skip(len);
-   return s;
+   return xstring::get_tmp(s,len);
 }
 const char *SFtp::lc_to_utf8(const char *s)
 {
-   if(!send_translate)
+   if(!send_translate || !s)
       return s;
 
    send_translate->ResetTranslation();
@@ -1934,7 +1942,7 @@ const char *SFtp::lc_to_utf8(const char *s)
    int len;
    send_translate->Get(&s,&len);
    send_translate->Skip(len);
-   return s;
+   return xstring::get_tmp(s,len);
 }
 
 FileSet *SFtp::GetFileSet()
@@ -1947,6 +1955,10 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 {
    const FileAttrs *a=&na->attrs;
    const char *name=utf8_to_lc(na->name);
+   const char *longname=utf8_to_lc(na->longname);
+
+   LogNote(10,"NameAttrs(name=\"%s\",type=%d,longname=\"%s\")\n",name?name:"",a->type,longname?longname:"");
+
    if(!name || !name[0])
       return 0;
    if(strchr(name,'/'))
@@ -1959,10 +1971,11 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
    case SSH_FILEXFER_TYPE_REGULAR:  fi->SetType(fi->NORMAL);    break;
    case SSH_FILEXFER_TYPE_DIRECTORY:fi->SetType(fi->DIRECTORY); break;
    case SSH_FILEXFER_TYPE_SYMLINK:  fi->SetType(fi->SYMLINK);   break;
+   case SSH_FILEXFER_TYPE_UNKNOWN: break;
    default: return 0;
    }
-   if(na->longname)
-      fi->SetLongName(utf8_to_lc(na->longname));
+   if(longname)
+      fi->SetLongName(longname);
    if(a->flags&SSH_FILEXFER_ATTR_SIZE)
       fi->SetSize(a->size);
    if(a->flags&SSH_FILEXFER_ATTR_UIDGID)
