@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.h,v 1.5 2006/12/27 13:56:02 lav Exp $ */
+/* $Id: Speedometer.h,v 1.7 2009/03/20 10:23:40 lav Exp $ */
 
 #ifndef SPEEDOMETER_H
 #define SPEEDOMETER_H
@@ -35,18 +35,16 @@ class Speedometer : public SMTask, public ResClient
    Time start;
    bool terse;
    const char *period_resource;
-   static char buf_eta[];
-   static char buf_rate[];
 public:
    Speedometer(const char *p="xfer:rate-period");
    float Get();
-   static const char *GetStr(float r);
-   const char *GetStr() { return GetStr(Get()); }
+   static xstring& GetStr(float r);
+   xstring& GetStr() { return GetStr(Get()); }
    static const char *GetStrS(float r);
    const char *GetStrS() { return GetStrS(Get()); }
-   const char *GetETAStrFromSize(off_t s);
+   xstring& GetETAStrFromSize(off_t s);
    const char *GetETAStrSFromSize(off_t s);
-   const char *GetETAStrFromTime(long t);
+   xstring& GetETAStrFromTime(long t);
    const char *GetETAStrSFromTime(long t);
    bool Valid();
    void Add(int bytes);
