@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,13 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: bookmark.cc,v 1.17 2006/02/06 10:57:28 lav Exp $ */
+/* $Id: bookmark.cc,v 1.19 2008/11/27 05:56:33 lav Exp $ */
 
 #include <config.h>
 #include "bookmark.h"
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stddef.h>
 #include "trio.h"
 #include "ResMgr.h"
 #include "misc.h"
@@ -38,14 +39,9 @@ ResDecl res_auto_sync("bmk:auto-sync","yes",ResMgr::BoolValidate,ResMgr::NoClosu
 
 Bookmark::Bookmark()
 {
-   bm_file=0;
    const char *home=get_lftp_home();
    if(home)
-   {
-      const char *add="/bookmarks";
-      bm_file=xstrdup(home,+strlen(add));
-      strcat(bm_file,add);
-   }
+      bm_file.vset(home,"/bookmarks",NULL);
    bm_fd=-1;
    stamp=(time_t)-1;
 }
@@ -53,7 +49,6 @@ Bookmark::Bookmark()
 Bookmark::~Bookmark()
 {
    Close();
-   xfree(bm_file);
 }
 
 void Bookmark::Refresh()
@@ -80,7 +75,7 @@ void Bookmark::Load()
 	 return;
       fcntl(bm_fd,F_SETFD,FD_CLOEXEC);
       if(Lock(F_RDLCK)==-1)
-	 fprintf(stderr,"%s: lock for reading failed, trying to read anyway\n",bm_file);
+	 fprintf(stderr,"%s: lock for reading failed, trying to read anyway\n",bm_file.get());
    }
    struct stat st;
    fstat(bm_fd,&st);
@@ -106,7 +101,7 @@ void Bookmark::PreModify()
       return;
    if(Lock(F_WRLCK)==-1)
    {
-      fprintf(stderr,"%s: lock for writing failed\n",bm_file);
+      fprintf(stderr,"%s: lock for writing failed\n",bm_file.get());
       Close();
       return;
    }
@@ -142,7 +137,7 @@ void Bookmark::UserSave()
       return;
    if(Lock(F_WRLCK)==-1)
    {
-      fprintf(stderr,"%s: lock for writing failed\n",bm_file);
+      fprintf(stderr,"%s: lock for writing failed\n",bm_file.get());
       Close();
       return;
    }
