@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: bookmark.h,v 1.5 2004/05/28 11:48:12 lav Exp $ */
+/* $Id: bookmark.h,v 1.8 2008/11/27 05:56:33 lav Exp $ */
 
 #ifndef BOOKMARK_H
 #define BOOKMARK_H
@@ -28,7 +28,7 @@
 
 class Bookmark : public KeyValueDB
 {
-   char *bm_file;
+   xstring bm_file;
    int bm_fd;
    time_t stamp;
 
@@ -39,12 +39,15 @@ class Bookmark : public KeyValueDB
    void PreModify();
    void PostModify();
    void Close();
+   void AutoSync();
 public:
    void Add(const char *id,const char *value);
    void Remove(const char *id);
    const char *Lookup(const char *id);
    char *Format();
    char *FormatHidePasswords();
+   void UserLoad() { Load(); Close(); }
+   void UserSave();
 
    Bookmark();
    ~Bookmark();
