@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2009 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.288 2008/05/22 13:20:38 lav Exp $ */
+/* $Id: commands.cc,v 1.294 2009/03/20 12:27:03 lav Exp $ */
 
 #include <config.h>
 
@@ -31,6 +31,7 @@
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <assert.h>
 #ifdef HAVE_DLFCN_H
@@ -642,6 +643,7 @@ Job *CmdExec::builtin_cd()
 
 Job *CmdExec::builtin_exit()
 {
+   bool detach=ResMgr::QueryBool("cmd:move-background-detach",0);
    bool bg=false;
    bool kill=false;
    int code=prev_exit_code;
@@ -670,7 +672,7 @@ Job *CmdExec::builtin_exit()
    }
    // Note: one job is this CmdExec.
    if(!bg && exec->top_level
-   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>1)
+   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>0)
    {
       eprintf(_(
 	 "There are running jobs and `cmd:move-background' is not set.\n"
@@ -678,9 +680,37 @@ Job *CmdExec::builtin_exit()
       ));
       return 0;
    }
+   if(!detach && Job::NumberOfJobs()==0)
+      detach=true;
    if(kill)
       Job::KillAll();
-   exec->Exit(code);
+   if(detach)
+      exec->Exit(code);
+   else {
+      int loc=0;
+      exec->SetAutoTerminateInBackground(true);
+      eprintf(_(
+	 "\n"
+	 "lftp now tricks the shell to move it to background process group.\n"
+	 "lftp continues to run in the background despite the `Stopped' message.\n"
+	 "lftp will automatically terminate when all jobs are finished.\n"
+	 "Use `fg' shell command to return to lftp if it is still running.\n"
+      ));
+      // trick the shell
+      switch(pid_t pid=fork()) {
+      case 0: // child
+	 sleep(1);   // wait for the parent to stop (is there a safer way?)
+	 ::kill(getppid(),SIGCONT);
+	 _exit(0);
+      default: // parent
+	 raise(SIGSTOP);
+	 waitpid(pid,&loc,0); // clean-up
+	 break;
+      case -1:
+	 exec->Exit(code);
+	 break;
+      }
+   }
    exit_code=code;
    return 0;
 }
@@ -1230,7 +1260,7 @@ Job *CmdExec::builtin_queue()
 
 	 if(!arg)
 	    queue->queue_feeder->DelJob(-1, verbose); /* delete the last job */
-	 else if(isdigit(arg[0]) && atoi(arg) != 0)
+	 else if(atoi(arg) != 0)
 	    queue->queue_feeder->DelJob(atoi(arg)-1, verbose);
 	 else
 	    queue->queue_feeder->DelJob(arg, verbose);
@@ -1245,7 +1275,7 @@ Job *CmdExec::builtin_queue()
 	  * queue -m "*get*" 1
 	  * queue -m 3    (move entry 3 to the end) */
          const char *a1 = args->getarg(args->getindex());
-	 if(a1 && !isdigit(a1[0])) {
+	 if(a1 && !isdigit((unsigned char)a1[0])) {
 	    eprintf(_("%s: -m: Number expected as second argument. "), args->a0());
 	    goto err;
 	 }
@@ -1258,7 +1288,7 @@ Job *CmdExec::builtin_queue()
 	    break;
 	 }
 
-	 if(isdigit(arg[0]) && atoi(arg) != 0) {
+	 if(atoi(arg) != 0) {
 	    queue->queue_feeder->MoveJob(atoi(arg)-1, to, verbose);
 	    break;
 	 }
@@ -1484,9 +1514,9 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 	 human_opts=human_autoscale|human_SI;
 	 break;
       case OPT_BLOCK_SIZE:
-	 if(!isdigit(optarg[0]))
-	    return _("invalid block size");
 	 output_block_size = atoi(optarg);
+	 if(output_block_size == 0)
+	    return _("invalid block size");
 	 break;
       case('1'):
 	 single_column = true;
@@ -2540,15 +2570,24 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2008);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2009);
    printf("\n");
    printf(
-      _("LFTP is free software, covered by the GNU General Public License, and you are\n"
-      "welcome to change it and/or distribute copies of it under certain conditions.\n"
-      "There is absolutely no warranty for LFTP.  See COPYING for details.\n"));
+      _("LFTP is free software: you can redistribute it and/or modify\n"
+      "it under the terms of the GNU General Public License as published by\n"
+      "the Free Software Foundation, either version 3 of the License, or\n"
+      "(at your option) any later version.\n"
+      "\n"
+      "This program is distributed in the hope that it will be useful,\n"
+      "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+      "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+      "GNU General Public License for more details.\n"
+      "\n"
+      "You should have received a copy of the GNU General Public License\n"
+      "along with LFTP.  If not, see <http://www.gnu.org/licenses/>.\n"));
    printf("\n");
    printf(
-      _("Send bug reports and questions to <%s>.\n"),"lftp@uniyar.ac.ru");
+      _("Send bug reports and questions to the mailing list <%s>.\n"),"lftp@uniyar.ac.ru");
 
 #if defined(HAVE_DLOPEN) && defined(RTLD_DEFAULT)
    /* Show some of loaded libraries. Modules can load those libraries on
@@ -2922,12 +2961,12 @@ CMD(du)
 	 separate_dirs = true;
 	 break;
       case OPT_BLOCK_SIZE:
-	 if(!isdigit(optarg[0]) || atoi(optarg) == 0)
+	 blocksize = atoi(optarg);
+	 if(blocksize == 0)
 	 {
 	    eprintf(_("%s: invalid block size `%s'\n"),op,optarg);
 	    return 0;
 	 }
-	 blocksize = atoi(optarg);
 	 break;
       case OPT_EXCLUDE:
 	 exclude=optarg;
