@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.155 2008/02/17 08:19:25 lav Exp $ */
+/* $Id: ftpclass.h,v 1.161 2009/04/27 07:46:29 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -87,6 +87,7 @@ class Ftp : public NetAccess
       bool translation_activated;
       bool proxy_is_http; // true when connection was established via http proxy.
       bool may_show_password;
+      bool can_do_pasv;
 
       int multiline_code; // the code of multiline response.
       int sync_wait;	  // number of commands in flight.
@@ -107,12 +108,15 @@ class Ftp : public NetAccess
       bool rest_supported;
       bool site_chmod_supported;
       bool site_utime_supported;
+      bool site_utime2_supported;   // two-argument SITE UTIME
       bool pret_supported;
       bool utf8_supported;
       bool lang_supported;
       bool mlst_supported;
       bool clnt_supported;
       bool host_supported;
+      bool mfmt_supported;
+      bool epsv_supported;
 
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
@@ -206,6 +210,8 @@ class Ftp : public NetAccess
 	 OPTS_UTF8,
 	 LANG,
 	 SITE_UTIME,
+	 SITE_UTIME2,
+	 ALLO,
 	 QUOTED		// check response for any command submitted by QUOTE_CMD
 #if USE_SSL
 	 ,AUTH_TLS,PROT,SSCN,CCC
@@ -236,7 +242,7 @@ class Ftp : public NetAccess
       ~ExpectQueue();
 
       void Push(Expect *e);
-      void Push(Expect::expect_t e) { Push(new Expect(e)); }
+      void Push(Expect::expect_t e);
       Expect *Pop();
       Expect *FindLastCWD();
       int Count() { return count; }
@@ -318,13 +324,7 @@ private:
    // response_queue and switch to a state if necessary
    int	 ReceiveResp();
 
-   bool ProxyIsHttp()
-      {
-	 if(!proxy_proto)
-	    return false;
-	 return !strcmp(proxy_proto,"http")
-	     || !strcmp(proxy_proto,"https");
-      }
+   bool ProxyIsHttp();
    int	 http_proxy_status_code;
    // Send CONNECT method to http proxy.
    void	 HttpProxySendConnect();
@@ -393,6 +393,7 @@ private:
 
    bool use_telnet_iac;
 
+   const char *get_protect_res();
    const char *encode_eprt(const sockaddr_u *);
 
    typedef FileInfo *(*FtpLineParser)(char *line,int *err,const char *tz);
