@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: history.h,v 1.3 2005/08/03 07:12:48 lav Exp $ */
+/* $Id: history.h,v 1.6 2008/11/27 05:56:37 lav Exp $ */
 
 #ifndef HISTORY_H
 #define HISTORY_H
@@ -31,7 +31,7 @@ class History : public KeyValueDB
    KeyValueDB *full;
    time_t stamp;
 
-   char *file;
+   xstring file;
    int fd;
    bool modified;
 
@@ -43,9 +43,9 @@ class History : public KeyValueDB
    time_t extract_stamp(const char *res);
 
 public:
-   void Set(FileAccess *s,const FileAccess::Path &p);
-   void Set(FileAccess *s) { Set(s,s->GetCwd()); }
-   const char *Lookup(FileAccess *s);
+   void Set(const FileAccess *s,const FileAccess::Path &p);
+   void Set(const FileAccess *s) { Set(s,s->GetCwd()); }
+   const char *Lookup(const FileAccess *s);
    void Save();
    History();
    ~History();
