@@ -1,11 +1,11 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -34,31 +34,19 @@
 #include "misc.h"
 
 lftp_ssl_base::lftp_ssl_base(int fd1,handshake_mode_t m,const char *h)
+   : hostname(h)
 {
    fd=fd1;
-   hostname=xstrdup(h);
    handshake_done=false;
    handshake_mode=m;
-   error=0;
    fatal=false;
 }
-lftp_ssl_base::~lftp_ssl_base()
-{
-   xfree(hostname);
-   xfree(error);
-}
 void lftp_ssl_base::set_error(const char *s1,const char *s2)
 {
-   xfree(error);
-   error=(char*)xmalloc(xstrlen(s1)+2+xstrlen(s2)+1);
-   if(s1)
-   {
-      strcpy(error,s1);
-      strcat(error,": ");
-      strcat(error,s2);
-   }
+   if(s2)
+      error.vset(s1,": ",s2,NULL);
    else
-      strcpy(error,s2);
+      error.set(s1);
 }
 void lftp_ssl_base::set_cert_error(const char *s)
 {
@@ -147,7 +135,12 @@ void lftp_ssl_gnutls_instance::LoadCA()
       res=gnutls_x509_crt_list_import(ca_list,&ca_list_size,&ca_pem,GNUTLS_X509_FMT_PEM,0);
    }
    if(res<0)
+   {
       Log::global->Format(0,"gnutls_x509_crt_list_import: %s\n",gnutls_strerror(res));
+      xfree(ca_list);
+      ca_list=0;
+      ca_list_size=0;
+   }
 
    munmap_file(ca_pem);
 }
@@ -174,7 +167,12 @@ void lftp_ssl_gnutls_instance::LoadCRL()
    crl_list=(gnutls_x509_crl_t*)xmalloc(crl_list_size*sizeof(gnutls_x509_crl_t));
    int res=gnutls_x509_crl_import(crl_list[0],&crl_pem,GNUTLS_X509_FMT_PEM);
    if(res<0)
+   {
       Log::global->Format(0,"gnutls_x509_crl_import: %s\n",gnutls_strerror(res));
+      xfree(crl_list);
+      crl_list=0;
+      crl_list_size=0;
+   }
 
    munmap_file(crl_pem);
 }
@@ -256,9 +254,15 @@ lftp_ssl_gnutls::lftp_ssl_gnutls(int fd1,handshake_mode_t m,const char *h)
 {
    global_init();
 
+   cred=0;
+
    gnutls_init(&session,(m==CLIENT?GNUTLS_CLIENT:GNUTLS_SERVER));
    gnutls_set_default_priority(session);
 
+   gnutls_transport_set_ptr(session,(gnutls_transport_ptr_t)fd);
+}
+void lftp_ssl_gnutls::load_keys()
+{
    gnutls_certificate_allocate_credentials(&cred);
    int res;
 #if 0
@@ -288,12 +292,16 @@ lftp_ssl_gnutls::lftp_ssl_gnutls(int fd1,handshake_mode_t m,const char *h)
 	 Log::global->Format(0,"gnutls_certificate_set_x509_key_file(%s,%s): %s\n",cert_file,key_file,gnutls_strerror(res));
    }
    gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, cred);
-
-   gnutls_transport_set_ptr(session,(gnutls_transport_ptr_t)fd);
+}
+void lftp_ssl_gnutls::shutdown()
+{
+   if(handshake_done)
+      gnutls_bye(session,GNUTLS_SHUT_RDWR);  // FIXME - E_AGAIN
 }
 lftp_ssl_gnutls::~lftp_ssl_gnutls()
 {
-   gnutls_certificate_free_credentials(cred);
+   if(cred)
+      gnutls_certificate_free_credentials(cred);
    gnutls_deinit(session);
 }
 
@@ -327,11 +335,11 @@ void lftp_ssl_gnutls::verify_certificate_chain(const gnutls_datum_t *cert_chain,
 
    /* Check if the name in the first certificate matches our destination!
     */
-   if(!gnutls_x509_crt_check_hostname(cert[0], hostname))
+   bool check_hostname = ResMgr::QueryBool("ssl:check-hostname", hostname);
+   if(check_hostname && !gnutls_x509_crt_check_hostname(cert[0], hostname))
    {
-      char *err=xasprintf("The certificate's owner does not match hostname '%s'\n",hostname);
+      xstring_ca err(xasprintf("The certificate's owner does not match hostname '%s'\n",hostname.get()));
       set_cert_error(err);
-      xfree(err);
    }
 
    for (i = 0; i < cert_chain_length; i++)
@@ -500,6 +508,7 @@ int lftp_ssl_gnutls::do_handshake()
       }
    }
    handshake_done=true;
+   SMTask::current->Timeout(0);
 
    if(gnutls_certificate_type_get(session)!=GNUTLS_CRT_X509)
    {
@@ -529,6 +538,11 @@ int lftp_ssl_gnutls::read(char *buf,int size)
    {
       if(res==GNUTLS_E_AGAIN || res==GNUTLS_E_INTERRUPTED)
 	 return RETRY;
+      else if(res==GNUTLS_E_UNEXPECTED_PACKET_LENGTH)
+      {
+	 Log::global->Format(7,"gnutls_record_recv: %s; assuming EOF\n",gnutls_strerror(res));
+	 return 0;
+      }
       else // error
       {
 	 fatal=check_fatal(res);
@@ -545,6 +559,8 @@ int lftp_ssl_gnutls::write(const char *buf,int size)
    int res=do_handshake();
    if(res!=DONE)
       return res;
+   if(size==0)
+      return 0;
    errno=0;
    res=gnutls_record_send(session,buf,size);
    if(res<0)
@@ -788,7 +804,9 @@ lftp_ssl_openssl::lftp_ssl_openssl(int fd1,handshake_mode_t m,const char *h)
    ssl=SSL_new(instance->ssl_ctx);
    SSL_set_fd(ssl,fd);
    SSL_ctrl(ssl,SSL_CTRL_MODE,SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER,0);
-
+}
+void lftp_ssl_openssl::load_keys()
+{
    const char *key_file =ResMgr::Query("ssl:key-file",hostname);
    const char *cert_file=ResMgr::Query("ssl:cert-file",hostname);
    if(key_file && !*key_file)
@@ -814,6 +832,11 @@ lftp_ssl_openssl::lftp_ssl_openssl(int fd1,handshake_mode_t m,const char *h)
       }
    }
 }
+void lftp_ssl_openssl::shutdown()
+{
+   if(handshake_done)
+      SSL_shutdown(ssl);
+}
 lftp_ssl_openssl::~lftp_ssl_openssl()
 {
    SSL_free(ssl);
@@ -835,7 +858,7 @@ bool lftp_ssl_openssl::check_fatal(int res)
 
 int lftp_ssl_openssl::do_handshake()
 {
-   if(SSL_is_init_finished(ssl))
+   if(handshake_done)
       return DONE;
    if(handshake_mode==SERVER)
    {
@@ -857,6 +880,8 @@ int lftp_ssl_openssl::do_handshake()
 	 return ERROR;
       }
    }
+   handshake_done=true;
+   SMTask::current->Timeout(0);
    return DONE;
 }
 int lftp_ssl_openssl::read(char *buf,int size)
@@ -886,6 +911,8 @@ int lftp_ssl_openssl::write(const char *buf,int size)
    int res=do_handshake();
    if(res!=DONE)
       return res;
+   if(size==0)
+      return 0;
    errno=0;
    res=SSL_write(ssl,buf,size);
    if(res<0)
