@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -34,8 +34,8 @@ public:
    int	 Done() { return session->IsClosed(); }
    int	 ExitCode() { return failed; }
 
-   void	 PrintStatus(int);
-   void	 ShowRunStatus(StatusLine *);
+   void	 PrintStatus(int,const char *);
+   void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
    void	 SayFinal();
 
    mvJob(FileAccess *session,const char *from,const char *to);
